@@ -14,19 +14,21 @@ package che
 import (
 	"context"
 	"fmt"
+	"reflect"
 	"strconv"
 	"strings"
 	"time"
 
-	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
-	"github.com/eclipse/che-operator/pkg/deploy"
-	devfile_registry "github.com/eclipse/che-operator/pkg/deploy/devfile-registry"
-	"github.com/eclipse/che-operator/pkg/deploy/gateway"
-	identity_provider "github.com/eclipse/che-operator/pkg/deploy/identity-provider"
-	plugin_registry "github.com/eclipse/che-operator/pkg/deploy/plugin-registry"
-	"github.com/eclipse/che-operator/pkg/deploy/postgres"
-	"github.com/eclipse/che-operator/pkg/deploy/server"
-	"github.com/eclipse/che-operator/pkg/util"
+	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	"github.com/eclipse-che/che-operator/pkg/deploy"
+	devworkspace "github.com/eclipse-che/che-operator/pkg/deploy/dev-workspace"
+	devfile_registry "github.com/eclipse-che/che-operator/pkg/deploy/devfile-registry"
+	"github.com/eclipse-che/che-operator/pkg/deploy/gateway"
+	identity_provider "github.com/eclipse-che/che-operator/pkg/deploy/identity-provider"
+	plugin_registry "github.com/eclipse-che/che-operator/pkg/deploy/plugin-registry"
+	"github.com/eclipse-che/che-operator/pkg/deploy/postgres"
+	"github.com/eclipse-che/che-operator/pkg/deploy/server"
+	"github.com/eclipse-che/che-operator/pkg/util"
 	configv1 "github.com/openshift/api/config/v1"
 	oauthv1 "github.com/openshift/api/config/v1"
 	consolev1 "github.com/openshift/api/console/v1"
@@ -34,12 +36,13 @@ import (
 	routev1 "github.com/openshift/api/route/v1"
 	userv1 "github.com/openshift/api/user/v1"
 	"github.com/sirupsen/logrus"
+	admissionregistrationv1 "k8s.io/api/admissionregistration/v1"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/api/extensions/v1beta1"
 	rbac "k8s.io/api/rbac/v1"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/discovery"
@@ -81,6 +84,7 @@ func newReconciler(mgr manager.Manager) (reconcile.Reconciler, error) {
 		nonCachedClient:   noncachedClient,
 		scheme:            mgr.GetScheme(),
 		discoveryClient:   discoveryClient,
+		userHandler:       NewOpenShiftOAuthUserHandler(noncachedClient),
 		permissionChecker: &K8sApiPermissionChecker{},
 	}, nil
 }
@@ -132,13 +136,23 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		if err := corev1.AddToScheme(mgr.GetScheme()); err != nil {
 			logrus.Errorf("Failed to add OpenShift Core to scheme: %s", err)
 		}
-		if hasConsolelinkObject() {
+		if util.HasAPIResourceName(deploy.ConsoleLinksResourceName) {
 			if err := consolev1.AddToScheme(mgr.GetScheme()); err != nil {
 				logrus.Errorf("Failed to add OpenShift ConsoleLink to scheme: %s", err)
 			}
 		}
 	}
 
+	// register Extension in the scheme
+	if err := apiextensionsv1.AddToScheme(mgr.GetScheme()); err != nil {
+		logrus.Errorf("Failed to add Extension to scheme: %s", err)
+	}
+
+	// register Admission in the scheme
+	if err := admissionregistrationv1.AddToScheme(mgr.GetScheme()); err != nil {
+		logrus.Errorf("Failed to add Admission to scheme: %s", err)
+	}
+
 	// register RBAC in the scheme
 	if err := rbac.AddToScheme(mgr.GetScheme()); err != nil {
 		logrus.Errorf("Failed to add RBAC to scheme: %s", err)
@@ -263,7 +277,6 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 var (
 	_ reconcile.Reconciler = &ReconcileChe{}
 
-	oAuthFinalizerName                           = "oauthclients.finalizers.che.eclipse.org"
 	cheWorkspacesClusterPermissionsFinalizerName = "cheWorkspaces.clusterpermissions.finalizers.che.eclipse.org"
 
 	// CheServiceAccountName - service account name for che-server.
@@ -284,17 +297,21 @@ type ReconcileChe struct {
 	// in the API Server
 	discoveryClient   discovery.DiscoveryInterface
 	scheme            *runtime.Scheme
-	permissionChecker PermissionChecker
 	tests             bool
+	userHandler       OpenShiftOAuthUserHandler
+	permissionChecker PermissionChecker
 }
 
 const (
 	failedValidationReason            = "InstallOrUpdateFailed"
-	failedNoOpenshiftUserReason       = "InstallOrUpdateFailed"
-	warningNoIdentityProvidersMessage = "No Openshift identity providers. Openshift oAuth was disabled. How to add identity provider read in the Help Link:"
-	warningNoRealUsersMessage         = "No real users. Openshift oAuth was disabled. How to add new user read in the Help Link:"
-	failedUnableToGetOAuth            = "Unable to get openshift oauth."
-	failedUnableToGetOpenshiftUsers   = "Unable to get users on the OpenShift cluster."
+	failedNoOpenshiftUser             = "NoOpenshiftUsers"
+	failedNoIdentityProviders         = "NoIdentityProviders"
+	failedUnableToGetOAuth            = "UnableToGetOpenshiftOAuth"
+	warningNoIdentityProvidersMessage = "No Openshift identity providers."
+
+	AddIdentityProviderMessage      = "Openshift oAuth was disabled. How to add identity provider read in the Help Link:"
+	warningNoRealUsersMessage       = "No real users. Openshift oAuth was disabled. How to add new user read in the Help Link:"
+	failedUnableToGetOpenshiftUsers = "Unable to get users on the OpenShift cluster."
 
 	howToAddIdentityProviderLinkOS4 = "https://docs.openshift.com/container-platform/latest/authentication/understanding-identity-provider.html#identity-provider-overview_understanding-identity-provider"
 	howToConfigureOAuthLinkOS3      = "https://docs.openshift.com/container-platform/3.11/install_config/configuring_authentication.html"
@@ -331,17 +348,8 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 		InternalService: deploy.InternalService{},
 	}
 
-	// delete oAuthClient before CR is deleted
-	// todo check
-	// instance.Status.OpenShiftoAuthProvisioned
-	if util.IsOpenShift && util.IsOAuthEnabled(instance) {
-		if err := r.ReconcileFinalizer(instance); err != nil {
-			return reconcile.Result{}, err
-		}
-	}
-	if r.reconcileWorkspacePermissionsFinalizer(instance, deployContext); err != nil {
-		return reconcile.Result{}, err
-	}
+	// Reconcile finalizers before CR is deleted
+	r.reconcileFinalizers(deployContext)
 
 	// Reconcile the imagePuller section of the CheCluster
 	imagePullerResult, err := deploy.ReconcileImagePuller(deployContext)
@@ -378,12 +386,58 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 		}
 	}
 
+	if isOpenShift4 && util.IsDeleteOAuthInitialUser(instance) {
+		if err := r.userHandler.DeleteOAuthInitialUser(deployContext); err != nil {
+			logrus.Errorf("Unable to delete initial OpenShift OAuth user from a cluster. Cause: %s", err.Error())
+			instance.Spec.Auth.InitialOpenShiftOAuthUser = nil
+			err := r.UpdateCheCRSpec(instance, "initialOpenShiftOAuthUser", "nil")
+			return reconcile.Result{}, err
+		}
+
+		instance.Spec.Auth.OpenShiftoAuth = nil
+		instance.Spec.Auth.InitialOpenShiftOAuthUser = nil
+		updateFields := map[string]string{
+			"openShiftoAuth":            "nil",
+			"initialOpenShiftOAuthUser": "nil",
+		}
+
+		if err := r.UpdateCheCRSpecByFields(instance, updateFields); err != nil {
+			return reconcile.Result{}, err
+		}
+
+		return reconcile.Result{}, nil
+	}
+
+	if instance.Spec.Auth.InitialOpenShiftOAuthUser == nil && instance.Status.OpenShiftOAuthUserCredentialsSecret != "" {
+		secret, err := deploy.GetSecret(deployContext, openShiftOAuthUserCredentialsSecret, instance.Namespace)
+		if secret == nil {
+			if err == nil {
+				instance.Status.OpenShiftOAuthUserCredentialsSecret = ""
+				if err := r.UpdateCheCRStatus(instance, "openShiftOAuthUserCredentialsSecret", ""); err != nil {
+					return reconcile.Result{}, err
+				}
+			} else {
+				return reconcile.Result{}, err
+			}
+		}
+	}
+
 	if isOpenShift && instance.Spec.Auth.OpenShiftoAuth == nil {
-		if reconcileResult, err := r.autoEnableOAuth(instance, request, isOpenShift4); err != nil {
+		if reconcileResult, err := r.autoEnableOAuth(deployContext, request, isOpenShift4); err != nil {
 			return reconcileResult, err
 		}
 	}
 
+	// Reconcile Dev Workspace Operator
+	done, err := devworkspace.ReconcileDevWorkspace(deployContext)
+	if !done {
+		if err != nil {
+			logrus.Error(err)
+		}
+		// We should `Requeue` since we don't watch Dev Workspace controller objects
+		return reconcile.Result{RequeueAfter: time.Second}, err
+	}
+
 	// Read proxy configuration
 	proxy, err := r.getProxyConfiguration(instance)
 	if err != nil {
@@ -473,12 +527,12 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	}
 
 	// Make sure that CA certificates from all marked config maps are merged into single config map to be propageted to Che components
-	cm, err := deploy.SyncAdditionalCACertsConfigMapToCluster(deployContext)
+	done, err = deploy.SyncAdditionalCACertsConfigMapToCluster(deployContext)
 	if err != nil {
 		logrus.Errorf("Error updating additional CA config map: %v", err)
 		return reconcile.Result{}, err
 	}
-	if cm == nil && !tests {
+	if !done && !tests {
 		// Config map update is in progress
 		// Return and do not force reconcile. When update finishes it will trigger reconcile loop.
 		return reconcile.Result{}, err
@@ -562,12 +616,13 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	}
 
 	if !util.IsOAuthEnabled(instance) && !util.IsWorkspaceInSameNamespaceWithChe(instance) {
-		clusterRole, err := deploy.GetClusterRole(CheWorkspacesClusterRoleNameTemplate, deployContext.ClusterAPI.Client)
+		сheWorkspacesClusterRoleName := fmt.Sprintf(CheWorkspacesClusterRoleNameTemplate, instance.Namespace)
+		exists, err := deploy.Get(deployContext, types.NamespacedName{Name: сheWorkspacesClusterRoleName}, &rbac.ClusterRole{})
 		if err != nil {
 			logrus.Error(err)
 			return reconcile.Result{RequeueAfter: time.Second}, err
 		}
-		if clusterRole == nil {
+		if !exists {
 			policies := append(getCheWorkspacesNamespacePolicy(), getCheWorkspacesPolicy()...)
 			deniedRules, err := r.permissionChecker.GetNotPermittedPolicyRules(policies, "")
 			if err != nil {
@@ -580,13 +635,13 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 				logrus.Warnf("Fall back to '%s' namespace for workspaces.", instance.Namespace)
 				delete(instance.Spec.Server.CustomCheProperties, "CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT")
 				instance.Spec.Server.WorkspaceNamespaceDefault = instance.Namespace
-				r.UpdateCheCRSpec(instance, "Default namespace for workspaces", instance.Namespace);
+				err := r.UpdateCheCRSpec(instance, "Default namespace for workspaces", instance.Namespace)
 				if err != nil {
 					logrus.Error(err)
 					return reconcile.Result{RequeueAfter: time.Second * 1}, err
 				}
 			} else {
-				reconcileResult, err := r.delegateWorkspacePermissionsInTheDifferNamespaceThanChe(instance, deployContext)
+				reconcileResult, err := r.delegateWorkspacePermissionsInTheDifferNamespaceThanChe(deployContext)
 				if err != nil {
 					logrus.Error(err)
 					return reconcile.Result{RequeueAfter: time.Second}, err
@@ -612,14 +667,14 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 		cheClusterRoles := strings.Split(instance.Spec.Server.CheClusterRoles, ",")
 		for _, cheClusterRole := range cheClusterRoles {
 			cheClusterRole := strings.TrimSpace(cheClusterRole)
-			cheClusterRoleBindingName := instance.Namespace + "-che-" + cheClusterRole
-			cheClusterRoleBinding, err := deploy.SyncClusterRoleBindingToCluster(deployContext, cheClusterRoleBindingName, "che", cheClusterRole)
-			if cheClusterRoleBinding == nil {
-				logrus.Infof("Waiting on cluster role binding '%s' to be created", cheClusterRoleBindingName)
-				if err != nil {
-					logrus.Error(err)
-				}
-				if !tests {
+			cheClusterRoleBindingName := cheClusterRole
+			done, err := deploy.SyncClusterRoleBindingAndAddFinalizerToCluster(deployContext, cheClusterRoleBindingName, CheServiceAccountName, cheClusterRole)
+			if !tests {
+				if !done {
+					logrus.Infof("Waiting on cluster role binding '%s' to be created", cheClusterRoleBindingName)
+					if err != nil {
+						logrus.Error(err)
+					}
 					return reconcile.Result{RequeueAfter: time.Second}, err
 				}
 			}
@@ -649,24 +704,34 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	cheMultiUser := deploy.GetCheMultiUser(instance)
 
 	if cheMultiUser == "false" {
-		pvcStatus := deploy.SyncPVCToCluster(deployContext, deploy.DefaultCheVolumeClaimName, "1Gi", cheFlavor)
+		done, err := deploy.SyncPVCToCluster(deployContext, deploy.DefaultCheVolumeClaimName, "1Gi", cheFlavor)
 		if !tests {
-			if !pvcStatus.Continue {
+			if !done {
 				logrus.Infof("Waiting on pvc '%s' to be bound. Sometimes PVC can be bound only when the first consumer is created.", deploy.DefaultCheVolumeClaimName)
-				if pvcStatus.Err != nil {
-					logrus.Error(pvcStatus.Err)
+				if err != nil {
+					logrus.Error(err)
 				}
-				return reconcile.Result{Requeue: pvcStatus.Requeue, RequeueAfter: time.Second * 1}, pvcStatus.Err
+				return reconcile.Result{}, err
 			}
 		}
 
-		if util.K8sclient.IsPVCExists(deploy.DefaultPostgresVolumeClaimName, instance.Namespace) {
-			util.K8sclient.DeletePVC(deploy.DefaultPostgresVolumeClaimName, instance.Namespace)
+		done, err = deploy.DeleteNamespacedObject(deployContext, deploy.DefaultPostgresVolumeClaimName, &corev1.PersistentVolumeClaim{})
+		if !tests {
+			if !done {
+				if err != nil {
+					logrus.Error(err)
+				}
+				return reconcile.Result{}, err
+			}
 		}
 	} else {
+		done, err := deploy.DeleteNamespacedObject(deployContext, deploy.DefaultCheVolumeClaimName, &corev1.PersistentVolumeClaim{})
 		if !tests {
-			if util.K8sclient.IsPVCExists(deploy.DefaultCheVolumeClaimName, instance.Namespace) {
-				util.K8sclient.DeletePVC(deploy.DefaultCheVolumeClaimName, instance.Namespace)
+			if !done {
+				if err != nil {
+					logrus.Error(err)
+				}
+				return reconcile.Result{}, err
 			}
 		}
 	}
@@ -675,8 +740,14 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	externalDB := instance.Spec.Database.ExternalDb
 	if !externalDB {
 		if cheMultiUser == "false" {
-			if util.K8sclient.IsDeploymentExists(deploy.PostgresName, instance.Namespace) {
-				util.K8sclient.DeleteDeployment(deploy.PostgresName, instance.Namespace)
+			done, err := deploy.Delete(deployContext, types.NamespacedName{Name: deploy.PostgresName, Namespace: instance.Namespace}, &appsv1.Deployment{})
+			if !tests {
+				if !done {
+					if err != nil {
+						logrus.Error(err)
+					}
+					return reconcile.Result{}, err
+				}
 			}
 		} else {
 			// Create a new postgres service
@@ -693,15 +764,15 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 			}
 
 			// Create a new Postgres PVC object
-			pvcStatus := deploy.SyncPVCToCluster(deployContext, deploy.DefaultPostgresVolumeClaimName, "1Gi", deploy.PostgresName)
+			done, err := deploy.SyncPVCToCluster(deployContext, deploy.DefaultPostgresVolumeClaimName, "1Gi", deploy.PostgresName)
 			if !tests {
-				if !pvcStatus.Continue {
+				if !done {
 					logrus.Infof("Waiting on pvc '%s' to be bound. Sometimes PVC can be bound only when the first consumer is created.", deploy.DefaultPostgresVolumeClaimName)
-					if pvcStatus.Err != nil {
-						logrus.Error(pvcStatus.Err)
+					if err != nil {
+						logrus.Error(err)
 					}
 
-					return reconcile.Result{Requeue: pvcStatus.Requeue, RequeueAfter: time.Second * 1}, pvcStatus.Err
+					return reconcile.Result{}, err
 				}
 			}
 
@@ -781,14 +852,13 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	exposedServiceName := getServerExposingServiceName(instance)
 	cheHost := ""
 	if !isOpenShift {
-		additionalLabels := deployContext.CheCluster.Spec.Server.CheServerIngress.Labels
 		ingress, err := deploy.SyncIngressToCluster(
 			deployContext,
 			cheFlavor,
 			instance.Spec.Server.CheHost,
 			exposedServiceName,
 			8080,
-			additionalLabels,
+			deployContext.CheCluster.Spec.Server.CheServerIngress,
 			cheFlavor)
 		if !tests {
 			if ingress == nil {
@@ -808,8 +878,14 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 			customHost = ""
 		}
 
-		additionalLabels := deployContext.CheCluster.Spec.Server.CheServerRoute.Labels
-		route, err := deploy.SyncRouteToCluster(deployContext, cheFlavor, customHost, exposedServiceName, 8080, additionalLabels, cheFlavor)
+		route, err := deploy.SyncRouteToCluster(
+			deployContext,
+			cheFlavor,
+			customHost,
+			exposedServiceName,
+			8080,
+			deployContext.CheCluster.Spec.Server.CheServerRoute,
+			cheFlavor)
 		if route == nil {
 			logrus.Infof("Waiting on route '%s' to be ready", cheFlavor)
 			if err != nil {
@@ -864,9 +940,9 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 
 	// create Che ConfigMap which is synced with CR and is not supposed to be manually edited
 	// controller will reconcile this CM with CR spec
-	cheConfigMap, err := server.SyncCheConfigMapToCluster(deployContext)
+	done, err = server.SyncCheConfigMapToCluster(deployContext)
 	if !tests {
-		if cheConfigMap == nil {
+		if !done {
 			logrus.Infof("Waiting on config map '%s' to be created", server.CheConfigMapName)
 			if err != nil {
 				logrus.Error(err)
@@ -890,16 +966,17 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 				logrus.Error(err)
 			}
 
-			deployment, err := r.GetEffectiveDeployment(instance, cheFlavor)
-			if err == nil {
-				if deployment.Status.AvailableReplicas < 1 {
+			cheDeployment := &appsv1.Deployment{}
+			exists, err := deploy.GetNamespacedObject(deployContext, cheFlavor, cheDeployment)
+			if exists {
+				if cheDeployment.Status.AvailableReplicas < 1 {
 					if instance.Status.CheClusterRunning != UnavailableStatus {
 						if err := r.SetCheUnavailableStatus(instance, request); err != nil {
 							instance, _ = r.GetCR(request)
 							return reconcile.Result{Requeue: true, RequeueAfter: time.Second * 1}, err
 						}
 					}
-				} else if deployment.Status.Replicas != 1 {
+				} else if cheDeployment.Status.Replicas != 1 {
 					if instance.Status.CheClusterRunning != RollingUpdateInProgressStatus {
 						if err := r.SetCheRollingUpdateStatus(instance, request); err != nil {
 							instance, _ = r.GetCR(request)
@@ -931,23 +1008,21 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	}
 
 	// we can now try to create consolelink, after che instance is available
-	if err := createConsoleLink(isOpenShift4, protocol, instance, r); err != nil {
-		logrus.Errorf("An error occurred during console link provisioning: %s", err)
-		return reconcile.Result{}, err
+	done, err = deploy.ReconcileConsoleLink(deployContext)
+	if !done {
+		if err != nil {
+			logrus.Error(err)
+		}
+		// We should `Requeue` since we created cluster object
+		return reconcile.Result{RequeueAfter: time.Second}, err
 	}
 
 	// Delete OpenShift identity provider if OpenShift oAuth is false in spec
 	// but OpenShiftoAuthProvisioned is true in CR status, e.g. when oAuth has been turned on and then turned off
 	deleted, err := r.ReconcileIdentityProvider(instance, isOpenShift4)
 	if deleted {
-		for {
-			if err := r.DeleteOAuthFinalizer(instance); err != nil &&
-				errors.IsConflict(err) {
-				instance, _ = r.GetCR(request)
-				continue
-			}
-			break
-		}
+		// ignore error
+		deploy.DeleteFinalizer(deployContext, deploy.OAuthFinalizerName)
 		for {
 			instance.Status.OpenShiftoAuthProvisioned = false
 			if err := r.UpdateCheCRStatus(instance, "status: provisioned with OpenShift identity provider", "false"); err != nil &&
@@ -972,72 +1047,6 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	return reconcile.Result{}, nil
 }
 
-func createConsoleLink(isOpenShift4 bool, protocol string, instance *orgv1.CheCluster, r *ReconcileChe) error {
-	if !isOpenShift4 || !hasConsolelinkObject() {
-		logrus.Debug("Console link won't be created. It's not supported by cluster")
-		// console link is supported only on OpenShift >= 4.2
-		return nil
-	}
-
-	if protocol != "https" {
-		logrus.Debug("Console link won't be created. It's not supported when http connection is used")
-		// console link is supported only with https
-		return nil
-	}
-	cheHost := instance.Spec.Server.CheHost
-	preparedConsoleLink := &consolev1.ConsoleLink{
-		ObjectMeta: metav1.ObjectMeta{
-			Name: deploy.DefaultConsoleLinkName(),
-			Annotations: map[string]string{
-				deploy.CheEclipseOrgNamespace: instance.Namespace,
-			},
-		},
-		Spec: consolev1.ConsoleLinkSpec{
-			Link: consolev1.Link{
-				Href: protocol + "://" + cheHost,
-				Text: deploy.DefaultConsoleLinkDisplayName()},
-			Location: consolev1.ApplicationMenu,
-			ApplicationMenu: &consolev1.ApplicationMenuSpec{
-				Section:  deploy.DefaultConsoleLinkSection(),
-				ImageURL: fmt.Sprintf("%s://%s%s", protocol, cheHost, deploy.DefaultConsoleLinkImage()),
-			},
-		},
-	}
-
-	existingConsoleLink := &consolev1.ConsoleLink{}
-
-	if getErr := r.nonCachedClient.Get(context.TODO(), client.ObjectKey{Name: deploy.DefaultConsoleLinkName()}, existingConsoleLink); getErr == nil {
-		// if found, update existing one. We need ResourceVersion from current one.
-		preparedConsoleLink.ResourceVersion = existingConsoleLink.ResourceVersion
-		logrus.Debugf("Updating the object: ConsoleLink, name: %s", existingConsoleLink.Name)
-		return r.nonCachedClient.Update(context.TODO(), preparedConsoleLink)
-	} else {
-		// if not found, create new one
-		if statusError, ok := getErr.(*errors.StatusError); ok &&
-			statusError.Status().Reason == metav1.StatusReasonNotFound {
-			logrus.Infof("Creating a new object: ConsoleLink, name: %s", preparedConsoleLink.Name)
-			return r.nonCachedClient.Create(context.TODO(), preparedConsoleLink)
-		} else {
-			return getErr
-		}
-	}
-}
-
-func hasConsolelinkObject() bool {
-	resourceList, err := util.GetServerResources()
-	if err != nil {
-		return false
-	}
-	for _, res := range resourceList {
-		for _, r := range res.APIResources {
-			if r.Name == "consolelinks" {
-				return true
-			}
-		}
-	}
-	return false
-}
-
 // EvaluateCheServerVersion evaluate che version
 // based on Checluster information and image defaults from env variables
 func EvaluateCheServerVersion(cr *orgv1.CheCluster) string {
@@ -1046,8 +1055,14 @@ func EvaluateCheServerVersion(cr *orgv1.CheCluster) string {
 
 func getDefaultCheHost(deployContext *deploy.DeployContext) (string, error) {
 	cheFlavor := deploy.DefaultCheFlavor(deployContext.CheCluster)
-	additionalLabels := deployContext.CheCluster.Spec.Server.CheServerRoute.Labels
-	route, err := deploy.SyncRouteToCluster(deployContext, cheFlavor, "", getServerExposingServiceName(deployContext.CheCluster), 8080, additionalLabels, cheFlavor)
+	route, err := deploy.SyncRouteToCluster(
+		deployContext,
+		cheFlavor,
+		"",
+		getServerExposingServiceName(deployContext.CheCluster),
+		8080,
+		deployContext.CheCluster.Spec.Server.CheServerRoute,
+		cheFlavor)
 	if route == nil {
 		logrus.Infof("Waiting on route '%s' to be ready", cheFlavor)
 		if err != nil {
@@ -1102,42 +1117,69 @@ func isTrustedBundleConfigMap(mgr manager.Manager, obj handler.MapObject) (bool,
 	}
 }
 
-func (r *ReconcileChe) autoEnableOAuth(cr *orgv1.CheCluster, request reconcile.Request, isOpenShift4 bool) (reconcile.Result, error) {
+func (r *ReconcileChe) autoEnableOAuth(deployContext *deploy.DeployContext, request reconcile.Request, isOpenShift4 bool) (reconcile.Result, error) {
 	var message, reason string
 	oauth := false
+	cr := deployContext.CheCluster
 	if isOpenShift4 {
-		oauthv1 := &oauthv1.OAuth{}
-		if err := r.nonCachedClient.Get(context.TODO(), types.NamespacedName{Name: "cluster"}, oauthv1); err != nil {
-			getOAuthV1ErrMsg := failedUnableToGetOAuth + " Cause: " + err.Error()
-			logrus.Errorf(getOAuthV1ErrMsg)
-			message = getOAuthV1ErrMsg
-			reason = failedNoOpenshiftUserReason
+		openshitOAuth, err := GetOpenshiftOAuth(deployContext.ClusterAPI.NonCachedClient)
+		if err != nil {
+			message = "Unable to get Openshift oAuth. Cause: " + err.Error()
+			logrus.Error(message)
+			reason = failedUnableToGetOAuth
 		} else {
-			oauth = len(oauthv1.Spec.IdentityProviders) >= 1
-			if !oauth {
-				logrus.Warn(warningNoIdentityProvidersMessage, " ", howToAddIdentityProviderLinkOS4)
+			if len(openshitOAuth.Spec.IdentityProviders) > 0 {
+				oauth = true
+			} else if util.IsInitialOpenShiftOAuthUserEnabled(cr) {
+				provisioned, err := r.userHandler.SyncOAuthInitialUser(openshitOAuth, deployContext)
+				if err != nil {
+					message = warningNoIdentityProvidersMessage + " Operator tried to create initial OpenShift OAuth user for HTPasswd identity provider, but failed. Cause: " + err.Error()
+					logrus.Error(message)
+					logrus.Info("To enable OpenShift OAuth, please add identity provider first: " + howToAddIdentityProviderLinkOS4)
+					reason = failedNoIdentityProviders
+					// Don't try to create initial user any more, che-operator shouldn't hang on this step.
+					cr.Spec.Auth.InitialOpenShiftOAuthUser = nil
+					if err := r.UpdateCheCRStatus(cr, "initialOpenShiftOAuthUser", ""); err != nil {
+						return reconcile.Result{}, err
+					}
+					oauth = false
+				} else {
+					if !provisioned {
+						return reconcile.Result{}, err
+					}
+					oauth = true
+					if deployContext.CheCluster.Status.OpenShiftOAuthUserCredentialsSecret == "" {
+						deployContext.CheCluster.Status.OpenShiftOAuthUserCredentialsSecret = openShiftOAuthUserCredentialsSecret
+						if err := r.UpdateCheCRStatus(cr, "openShiftOAuthUserCredentialsSecret", openShiftOAuthUserCredentialsSecret); err != nil {
+							return reconcile.Result{}, err
+						}
+					}
+				}
 			}
 		}
-		// openshift 3
-	} else {
+	} else { // Openshift 3
 		users := &userv1.UserList{}
 		listOptions := &client.ListOptions{}
 		if err := r.nonCachedClient.List(context.TODO(), users, listOptions); err != nil {
-			getUsersErrMsg := failedUnableToGetOpenshiftUsers + " Cause: " + err.Error()
-			logrus.Errorf(getUsersErrMsg)
-			message = getUsersErrMsg
-			reason = failedNoOpenshiftUserReason
+			message = failedUnableToGetOpenshiftUsers + " Cause: " + err.Error()
+			logrus.Error(message)
+			reason = failedNoOpenshiftUser
 		} else {
 			oauth = len(users.Items) >= 1
 			if !oauth {
-				logrus.Warn(warningNoRealUsersMessage, " ", howToConfigureOAuthLinkOS3)
+				message = warningNoRealUsersMessage + " " + howToConfigureOAuthLinkOS3
+				logrus.Warn(message)
+				reason = failedNoOpenshiftUser
 			}
 		}
 	}
 
-	cr.Spec.Auth.OpenShiftoAuth = util.NewBoolPointer(oauth)
-	if err := r.UpdateCheCRSpec(cr, "OpenShiftoAuth", strconv.FormatBool(oauth)); err != nil {
-		return reconcile.Result{Requeue: true, RequeueAfter: time.Second * 1}, err
+	newOAuthValue := util.NewBoolPointer(oauth)
+	if !reflect.DeepEqual(newOAuthValue, cr.Spec.Auth.OpenShiftoAuth) {
+		cr.Spec.Auth.OpenShiftoAuth = newOAuthValue
+		if err := r.UpdateCheCRSpec(cr, "openShiftoAuth", strconv.FormatBool(oauth)); err != nil {
+			return reconcile.Result{Requeue: true, RequeueAfter: time.Second * 1}, err
+		}
 	}
 
 	if message != "" && reason != "" {
@@ -1173,3 +1215,46 @@ func isEclipseCheSecret(mgr manager.Manager, obj handler.MapObject) (bool, recon
 		},
 	}
 }
+
+func (r *ReconcileChe) reconcileFinalizers(deployContext *deploy.DeployContext) {
+	if util.IsOpenShift && util.IsOAuthEnabled(deployContext.CheCluster) {
+		if err := deploy.ReconcileOAuthClientFinalizer(deployContext); err != nil {
+			logrus.Error(err)
+		}
+	}
+
+	if err := r.reconcileWorkspacePermissionsFinalizer(deployContext); err != nil {
+		logrus.Error(err)
+	}
+
+	if err := deploy.ReconcileConsoleLinkFinalizer(deployContext); err != nil {
+		logrus.Error(err)
+	}
+
+	if len(deployContext.CheCluster.Spec.Server.CheClusterRoles) > 0 {
+		cheClusterRoles := strings.Split(deployContext.CheCluster.Spec.Server.CheClusterRoles, ",")
+		for _, cheClusterRole := range cheClusterRoles {
+			cheClusterRole := strings.TrimSpace(cheClusterRole)
+			cheClusterRoleBindingName := cheClusterRole
+			if err := deploy.ReconcileClusterRoleBindingFinalizer(deployContext, cheClusterRoleBindingName); err != nil {
+				logrus.Error(err)
+			}
+
+			// Removes any legacy CRB https://github.com/eclipse/che/issues/19506
+			cheClusterRoleBindingName = deploy.GetLegacyUniqueClusterRoleBindingName(deployContext, CheServiceAccountName, cheClusterRole)
+			if err := deploy.ReconcileLegacyClusterRoleBindingFinalizer(deployContext, cheClusterRoleBindingName); err != nil {
+				logrus.Error(err)
+			}
+		}
+	}
+}
+
+func (r *ReconcileChe) GetCR(request reconcile.Request) (instance *orgv1.CheCluster, err error) {
+	instance = &orgv1.CheCluster{}
+	err = r.client.Get(context.TODO(), request.NamespacedName, instance)
+	if err != nil {
+		logrus.Errorf("Failed to get %s CR: %s", instance.Name, err)
+		return nil, err
+	}
+	return instance, nil
+}
