@@ -16,9 +16,9 @@ import (
 	"os"
 	"testing"
 
-	"github.com/eclipse/che-operator/pkg/util"
+	"github.com/eclipse-che/che-operator/pkg/util"
 
-	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
+	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
 )
 
 func TestDefaultFromEnv(t *testing.T) {
@@ -87,7 +87,7 @@ func TestCorrectImageName(t *testing.T) {
 	testCases := map[string]string{
 		"registry.redhat.io/codeready-workspaces/crw-2-rhel8-operator:latest":  "crw-2-rhel8-operator:latest",
 		"eclipse/che-operator:7.1.0":            "che-operator:7.1.0",
-		"registry.redhat.io/codeready-workspaces/crw-2-rhel8-operator:2.7":  "crw-2-rhel8-operator:2.7",
+		"registry.redhat.io/codeready-workspaces/crw-2-rhel8-operator:2.8":  "crw-2-rhel8-operator:2.8",
 	}
 	for k, v := range testCases {
 		t.Run(k, func(*testing.T) {
