@@ -18,9 +18,9 @@ import (
 	"strconv"
 	"strings"
 
-	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
-	"github.com/eclipse/che-operator/pkg/deploy"
-	"github.com/eclipse/che-operator/pkg/util"
+	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	"github.com/eclipse-che/che-operator/pkg/deploy"
+	"github.com/eclipse-che/che-operator/pkg/util"
 	"github.com/google/go-cmp/cmp"
 	"github.com/sirupsen/logrus"
 	appsv1 "k8s.io/api/apps/v1"
@@ -89,12 +89,15 @@ func GetSpecKeycloakDeployment(
 	}
 
 	if clusterDeployment != nil {
-		env := clusterDeployment.Spec.Template.Spec.Containers[0].Env
-		for _, e := range env {
-			// To be compatible with prev deployments when "TRUSTPASS" env was used
-			if "TRUSTPASS" == e.Name || "SSO_TRUSTSTORE_PASSWORD" == e.Name {
-				trustpass = e.Value
-				break
+		// To be compatible with prev deployments when "TRUSTPASS" env was used
+		clusterContainer := &clusterDeployment.Spec.Template.Spec.Containers[0]
+		env := util.FindEnv(clusterContainer.Env, "TRUSTPASS")
+		if env != nil {
+			trustpass = env.Value
+		} else {
+			env := util.FindEnv(clusterContainer.Env, "SSO_TRUSTSTORE_PASSWORD")
+			if env != nil {
+				trustpass = env.Value
 			}
 		}
 	}
@@ -477,6 +480,40 @@ func GetSpecKeycloakDeployment(
 		}
 	}
 
+	// Mount GITHUB_CLIENT_ID and GITHUB_SECRET to keycloak container
+	secrets, err := deploy.GetSecrets(deployContext, map[string]string{
+		deploy.KubernetesPartOfLabelKey:    deploy.CheEclipseOrg,
+		deploy.KubernetesComponentLabelKey: deploy.OAuthScmConfiguration,
+	}, map[string]string{
+		deploy.CheEclipseOrgOAuthScmServer: "github",
+	})
+
+	if err != nil {
+		return nil, err
+	} else if len(secrets) == 1 {
+		keycloakEnv = append(keycloakEnv, corev1.EnvVar{
+			Name: "GITHUB_CLIENT_ID",
+			ValueFrom: &corev1.EnvVarSource{
+				SecretKeyRef: &corev1.SecretKeySelector{
+					Key: "id",
+					LocalObjectReference: corev1.LocalObjectReference{
+						Name: secrets[0].Name,
+					},
+				},
+			},
+		}, corev1.EnvVar{
+			Name: "GITHUB_SECRET",
+			ValueFrom: &corev1.EnvVarSource{
+				SecretKeyRef: &corev1.SecretKeySelector{
+					Key: "secret",
+					LocalObjectReference: corev1.LocalObjectReference{
+						Name: secrets[0].Name,
+					},
+				},
+			},
+		})
+	}
+
 	for _, envvar := range proxyEnvVars {
 		keycloakEnv = append(keycloakEnv, envvar)
 	}
