@@ -1,3 +1,14 @@
+//
+// Copyright (c) 2020-2021 Red Hat, Inc.
+// This program and the accompanying materials are made
+// available under the terms of the Eclipse Public License 2.0
+// which is available at https://www.eclipse.org/legal/epl-2.0/
+//
+// SPDX-License-Identifier: EPL-2.0
+//
+// Contributors:
+//   Red Hat, Inc. - initial API and implementation
+//
 package deploy
 
 import (
@@ -5,8 +16,8 @@ import (
 	"time"
 
 	chev1alpha1 "github.com/che-incubator/kubernetes-image-puller-operator/pkg/apis/che/v1alpha1"
-	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
-	"github.com/eclipse/che-operator/pkg/util"
+	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	"github.com/eclipse-che/che-operator/pkg/util"
 	operatorsv1 "github.com/operator-framework/api/pkg/operators/v1"
 	operatorsv1alpha1 "github.com/operator-framework/api/pkg/operators/v1alpha1"
 	packagesv1 "github.com/operator-framework/operator-lifecycle-manager/pkg/package-server/apis/operators/v1"
@@ -183,13 +194,7 @@ func HasImagePullerFinalizer(instance *orgv1.CheCluster) bool {
 func ReconcileImagePullerFinalizer(ctx *DeployContext) (err error) {
 	instance := ctx.CheCluster
 	if instance.ObjectMeta.DeletionTimestamp.IsZero() {
-		if !util.ContainsString(instance.ObjectMeta.Finalizers, imagePullerFinalizerName) {
-			ctx.CheCluster.ObjectMeta.Finalizers = append(instance.ObjectMeta.Finalizers, imagePullerFinalizerName)
-			logrus.Infof("Adding finalizer %v", imagePullerFinalizerName)
-			if err := ctx.ClusterAPI.Client.Update(context.Background(), instance); err != nil {
-				return err
-			}
-		}
+		return AppendFinalizer(ctx, imagePullerFinalizerName)
 	} else {
 		if util.ContainsString(instance.ObjectMeta.Finalizers, imagePullerFinalizerName) {
 			clusterServiceVersionName := DefaultKubernetesImagePullerOperatorCSV()
@@ -204,17 +209,11 @@ func ReconcileImagePullerFinalizer(ctx *DeployContext) (err error) {
 				logrus.Errorf("Failed to delete %s ClusterServiceVersion: %s", clusterServiceVersionName, err)
 				return err
 			}
-			instance.ObjectMeta.Finalizers = util.DoRemoveString(instance.ObjectMeta.Finalizers, imagePullerFinalizerName)
-			logrus.Infof("Updating %s CR", instance.Name)
 
-			if err := ctx.ClusterAPI.Client.Update(context.Background(), instance); err != nil {
-				logrus.Errorf("Failed to update %s CR: %s", instance.Name, err)
-				return err
-			}
+			return DeleteFinalizer(ctx, imagePullerFinalizerName)
 		}
 		return nil
 	}
-	return nil
 }
 
 func DeleteImagePullerFinalizer(ctx *DeployContext) (err error) {
@@ -246,23 +245,27 @@ func SubscriptionsAreEqual(expected *operatorsv1alpha1.Subscription, actual *ope
 // foundKubernetesImagePullerAPI - true if the server discovers the che.eclipse.org API
 // error - any error returned by the call to discoveryClient.ServerGroups()
 func CheckNeededImagePullerApis(ctx *DeployContext) (bool, bool, bool, error) {
-	groupList, err := ctx.ClusterAPI.DiscoveryClient.ServerGroups()
+	groupList, resourcesList, err := ctx.ClusterAPI.DiscoveryClient.ServerGroupsAndResources()
 	if err != nil {
 		return false, false, false, err
 	}
-	groups := groupList.Groups
 	foundPackagesAPI := false
 	foundOperatorsAPI := false
 	foundKubernetesImagePullerAPI := false
-	for _, group := range groups {
+	for _, group := range groupList {
 		if group.Name == packagesv1.SchemeGroupVersion.Group {
 			foundPackagesAPI = true
 		}
 		if group.Name == operatorsv1alpha1.SchemeGroupVersion.Group {
 			foundOperatorsAPI = true
 		}
-		if group.Name == chev1alpha1.SchemeGroupVersion.Group {
-			foundKubernetesImagePullerAPI = true
+	}
+
+	for _, l := range resourcesList {
+		for _, r := range l.APIResources {
+			if l.GroupVersion == chev1alpha1.SchemeGroupVersion.String() && r.Kind == "KubernetesImagePuller" {
+				foundKubernetesImagePullerAPI = true
+			}
 		}
 	}
 	return foundPackagesAPI, foundOperatorsAPI, foundKubernetesImagePullerAPI, nil
