@@ -15,6 +15,7 @@ import (
 	"context"
 	"fmt"
 
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
 
 	"github.com/google/go-cmp/cmp"
@@ -23,6 +24,8 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	k8slabels "k8s.io/apimachinery/pkg/labels"
+	"k8s.io/apimachinery/pkg/selection"
 	"k8s.io/apimachinery/pkg/types"
 )
 
@@ -30,18 +33,19 @@ var secretDiffOpts = cmp.Options{
 	cmpopts.IgnoreFields(corev1.Secret{}, "TypeMeta", "ObjectMeta"),
 }
 
-// SyncSecretToCluster applies secret into cluster
-func SyncSecretToCluster(
+// SyncSecret applies secret into cluster or external namespace
+func SyncSecret(
 	deployContext *DeployContext,
 	name string,
+	namespace string,
 	data map[string][]byte) (*corev1.Secret, error) {
 
-	specSecret, err := GetSpecSecret(deployContext, name, data)
+	specSecret, err := GetSpecSecret(deployContext, name, namespace, data)
 	if err != nil {
 		return nil, err
 	}
 
-	clusterSecret, err := GetClusterSecret(specSecret.Name, specSecret.Namespace, deployContext.ClusterAPI)
+	clusterSecret, err := GetSecret(deployContext, specSecret.Name, specSecret.Namespace)
 	if err != nil {
 		return nil, err
 	}
@@ -71,14 +75,21 @@ func SyncSecretToCluster(
 	return clusterSecret, nil
 }
 
-// GetClusterSecret retrieves given secret from cluster
-func GetClusterSecret(name string, namespace string, clusterAPI ClusterAPI) (*corev1.Secret, error) {
+// GetSecret retrieves given secret from cluster
+func GetSecret(deployContext *DeployContext, name string, namespace string) (*corev1.Secret, error) {
 	secret := &corev1.Secret{}
 	namespacedName := types.NamespacedName{
 		Namespace: namespace,
 		Name:      name,
 	}
-	err := clusterAPI.Client.Get(context.TODO(), namespacedName, secret)
+
+	var err error
+	if namespace == deployContext.CheCluster.ObjectMeta.Namespace {
+		err = deployContext.ClusterAPI.Client.Get(context.TODO(), namespacedName, secret)
+	} else {
+		err = deployContext.ClusterAPI.NonCachedClient.Get(context.TODO(), namespacedName, secret)
+	}
+
 	if err != nil {
 		if errors.IsNotFound(err) {
 			return nil, nil
@@ -88,8 +99,48 @@ func GetClusterSecret(name string, namespace string, clusterAPI ClusterAPI) (*co
 	return secret, nil
 }
 
+// Get all secrets by labels and annotations
+func GetSecrets(deployContext *DeployContext, labels map[string]string, annotations map[string]string) ([]corev1.Secret, error) {
+	secrets := []corev1.Secret{}
+
+	labelSelector := k8slabels.NewSelector()
+	for k, v := range labels {
+		req, err := k8slabels.NewRequirement(k, selection.Equals, []string{v})
+		if err != nil {
+			return secrets, err
+		}
+		labelSelector = labelSelector.Add(*req)
+	}
+
+	listOptions := &client.ListOptions{
+		Namespace:     deployContext.CheCluster.Namespace,
+		LabelSelector: labelSelector,
+	}
+	secretList := &corev1.SecretList{}
+	if err := deployContext.ClusterAPI.Client.List(context.TODO(), secretList, listOptions); err != nil {
+		return secrets, err
+	}
+
+	for _, secret := range secretList.Items {
+		annotationsOk := true
+		for k, v := range annotations {
+			_, annotationExists := secret.Annotations[k]
+			if !annotationExists || secret.Annotations[k] != v {
+				annotationsOk = false
+				break
+			}
+		}
+
+		if annotationsOk {
+			secrets = append(secrets, secret)
+		}
+	}
+
+	return secrets, nil
+}
+
 // GetSpecSecret return default secret config for given data
-func GetSpecSecret(deployContext *DeployContext, name string, data map[string][]byte) (*corev1.Secret, error) {
+func GetSpecSecret(deployContext *DeployContext, name string, namespace string, data map[string][]byte) (*corev1.Secret, error) {
 	labels := GetLabels(deployContext.CheCluster, DefaultCheFlavor(deployContext.CheCluster))
 	secret := &corev1.Secret{
 		TypeMeta: metav1.TypeMeta{
@@ -98,15 +149,17 @@ func GetSpecSecret(deployContext *DeployContext, name string, data map[string][]
 		},
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      name,
-			Namespace: deployContext.CheCluster.Namespace,
+			Namespace: namespace,
 			Labels:    labels,
 		},
 		Data: data,
 	}
 
-	err := controllerutil.SetControllerReference(deployContext.CheCluster, secret, deployContext.ClusterAPI.Scheme)
-	if err != nil {
-		return nil, err
+	if deployContext.CheCluster.Namespace == namespace {
+		err := controllerutil.SetControllerReference(deployContext.CheCluster, secret, deployContext.ClusterAPI.Scheme)
+		if err != nil {
+			return nil, err
+		}
 	}
 
 	return secret, nil
@@ -124,7 +177,7 @@ func CreateTLSSecretFromEndpoint(deployContext *DeployContext, url string, name
 			return err
 		}
 
-		secret, err = SyncSecretToCluster(deployContext, name, map[string][]byte{"ca.crt": crtBytes})
+		secret, err = SyncSecret(deployContext, name, deployContext.CheCluster.Namespace, map[string][]byte{"ca.crt": crtBytes})
 		if err != nil {
 			return err
 		}
@@ -132,3 +185,24 @@ func CreateTLSSecretFromEndpoint(deployContext *DeployContext, url string, name
 
 	return nil
 }
+
+// DeleteSecret - delete secret by name and namespace
+func DeleteSecret(secretName string, namespace string, runtimeClient client.Client) error {
+	logrus.Infof("Delete secret: %s in the namespace: %s", secretName, namespace)
+
+	secret := &corev1.Secret{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      secretName,
+			Namespace: namespace,
+		},
+	}
+
+	if err := runtimeClient.Delete(context.TODO(), secret); err != nil {
+		if errors.IsNotFound(err) {
+			return nil
+		}
+		return err
+	}
+
+	return nil
+}
