@@ -14,12 +14,13 @@ package server
 import (
 	"encoding/json"
 	"fmt"
+	"net/url"
 	"os"
 	"strconv"
 
-	"github.com/eclipse/che-operator/pkg/deploy"
+	"github.com/eclipse-che/che-operator/pkg/deploy"
 
-	"github.com/eclipse/che-operator/pkg/util"
+	"github.com/eclipse-che/che-operator/pkg/util"
 	"github.com/sirupsen/logrus"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/labels"
@@ -84,19 +85,15 @@ type CheConfigMap struct {
 	ServerStrategy                         string `json:"CHE_INFRA_KUBERNETES_SERVER__STRATEGY"`
 	WorkspaceExposure                      string `json:"CHE_INFRA_KUBERNETES_SINGLEHOST_WORKSPACE_EXPOSURE"`
 	SingleHostGatewayConfigMapLabels       string `json:"CHE_INFRA_KUBERNETES_SINGLEHOST_GATEWAY_CONFIGMAP__LABELS"`
+	CheDevWorkspacesEnabled                string `json:"CHE_DEVWORKSPACES_ENABLED"`
 }
 
-func SyncCheConfigMapToCluster(deployContext *deploy.DeployContext) (*corev1.ConfigMap, error) {
+func SyncCheConfigMapToCluster(deployContext *deploy.DeployContext) (bool, error) {
 	data, err := GetCheConfigMapData(deployContext)
 	if err != nil {
-		return nil, err
+		return false, err
 	}
-	specConfigMap, err := deploy.GetSpecConfigMap(deployContext, CheConfigMapName, data, deploy.DefaultCheFlavor(deployContext.CheCluster))
-	if err != nil {
-		return nil, err
-	}
-
-	return deploy.SyncConfigMapToCluster(deployContext, specConfigMap)
+	return deploy.SyncConfigMapDataToCluster(deployContext, CheConfigMapName, data, deploy.DefaultCheFlavor(deployContext.CheCluster))
 }
 
 // GetCheConfigMapData gets env values from CR spec and returns a map with key:value
@@ -104,6 +101,9 @@ func SyncCheConfigMapToCluster(deployContext *deploy.DeployContext) (*corev1.Con
 func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string]string, err error) {
 	cheHost := deployContext.CheCluster.Spec.Server.CheHost
 	keycloakURL := deployContext.CheCluster.Spec.Auth.IdentityProviderURL
+	if err != nil {
+		logrus.Errorf("Failed to get current infra: %s", err)
+	}
 	cheFlavor := deploy.DefaultCheFlavor(deployContext.CheCluster)
 	infra := "kubernetes"
 	if util.IsOpenShift {
@@ -120,10 +120,8 @@ func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string
 	namespaceAllowUserDefined := strconv.FormatBool(deployContext.CheCluster.Spec.Server.AllowUserDefinedWorkspaceNamespaces)
 	tlsSupport := deployContext.CheCluster.Spec.Server.TlsSupport
 	protocol := "http"
-	wsprotocol := "ws"
 	if tlsSupport {
 		protocol = "https"
-		wsprotocol = "wss"
 		tls = "true"
 	}
 
@@ -174,7 +172,7 @@ func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string
 	workspaceNamespaceDefault := util.GetWorkspaceNamespaceDefault(deployContext.CheCluster)
 
 	cheAPI := protocol + "://" + cheHost + "/api"
-	var keycloakInternalURL, pluginRegistryInternalURL, devfileRegistryInternalURL, cheInternalAPI string
+	var keycloakInternalURL, pluginRegistryInternalURL, devfileRegistryInternalURL, cheInternalAPI, webSocketEndpoint, webSocketEndpointMinor string
 
 	if deployContext.CheCluster.Spec.Server.UseInternalClusterSVCNames && !deployContext.CheCluster.Spec.Auth.ExternalIdentityProvider {
 		keycloakInternalURL = deployContext.InternalService.KeycloakHost
@@ -196,8 +194,25 @@ func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string
 
 	if deployContext.CheCluster.Spec.Server.UseInternalClusterSVCNames {
 		cheInternalAPI = deployContext.InternalService.CheHost + "/api"
+
+		u, err := url.Parse(deployContext.InternalService.CheHost)
+		if err != nil {
+			return nil, err
+		}
+
+		webSocketEndpoint = "ws://" + u.Host + "/api/websocket"
+		webSocketEndpointMinor = "ws://" + u.Host + "/api/websocket-minor"
 	} else {
 		cheInternalAPI = cheAPI
+
+		wsprotocol := "ws"
+
+		if tlsSupport {
+			wsprotocol = "wss"
+		}
+
+		webSocketEndpoint = wsprotocol + "://" + cheHost + "/api/websocket"
+		webSocketEndpointMinor = wsprotocol + "://" + cheHost + "/api/websocket-minor"
 	}
 
 	data := &CheConfigMap{
@@ -206,8 +221,8 @@ func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string
 		ChePort:                                "8080",
 		CheApi:                                 cheAPI,
 		CheApiInternal:                         cheInternalAPI,
-		CheWebSocketEndpoint:                   wsprotocol + "://" + cheHost + "/api/websocket",
-		WebSocketEndpointMinor:                 wsprotocol + "://" + cheHost + "/api/websocket-minor",
+		CheWebSocketEndpoint:                   webSocketEndpoint,
+		WebSocketEndpointMinor:                 webSocketEndpointMinor,
 		CheDebugServer:                         cheDebug,
 		CheInfrastructureActive:                infra,
 		CheInfraKubernetesServiceAccountName:   "che-workspace",
@@ -242,6 +257,7 @@ func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string
 		ServerStrategy:                         ingressStrategy,
 		WorkspaceExposure:                      workspaceExposure,
 		SingleHostGatewayConfigMapLabels:       singleHostGatewayConfigMapLabels,
+		CheDevWorkspacesEnabled:                strconv.FormatBool(deployContext.CheCluster.Spec.DevWorkspace.Enable),
 	}
 
 	if cheMultiUser == "true" {
@@ -277,33 +293,57 @@ func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string
 		// Add TLS key and server certificate to properties when user workspaces should be created in another
 		// than Che server namespace, from where the Che TLS secret is not accessable
 		if !util.IsWorkspaceInSameNamespaceWithChe(deployContext.CheCluster) {
-			cheTLSSecret, err := deploy.GetClusterSecret(deployContext.CheCluster.Spec.K8s.TlsSecretName, deployContext.CheCluster.ObjectMeta.Namespace, deployContext.ClusterAPI)
-			if err != nil {
-				return nil, err
+			if deployContext.CheCluster.Spec.K8s.TlsSecretName != "" {
+				cheTLSSecret, err := deploy.GetSecret(deployContext, deployContext.CheCluster.Spec.K8s.TlsSecretName, deployContext.CheCluster.ObjectMeta.Namespace)
+				if err != nil {
+					return nil, err
+				}
+				if cheTLSSecret == nil {
+					return nil, fmt.Errorf("%s secret not found", deployContext.CheCluster.Spec.K8s.TlsSecretName)
+				} else {
+					if _, exists := cheTLSSecret.Data["tls.key"]; !exists {
+						return nil, fmt.Errorf("%s secret has no 'tls.key' key in data", deployContext.CheCluster.Spec.K8s.TlsSecretName)
+					}
+					if _, exists := cheTLSSecret.Data["tls.crt"]; !exists {
+						return nil, fmt.Errorf("%s secret has no 'tls.crt' key in data", deployContext.CheCluster.Spec.K8s.TlsSecretName)
+					}
+					k8sCheEnv["CHE_INFRA_KUBERNETES_TLS__KEY"] = string(cheTLSSecret.Data["tls.key"])
+					k8sCheEnv["CHE_INFRA_KUBERNETES_TLS__CERT"] = string(cheTLSSecret.Data["tls.crt"])
+				}
 			}
-			if cheTLSSecret == nil {
-				return nil, fmt.Errorf("%s secret not found", deployContext.CheCluster.Spec.K8s.TlsSecretName)
-			}
-			if _, exists := cheTLSSecret.Data["tls.key"]; !exists {
-				return nil, fmt.Errorf("%s secret has no 'tls.key' key in data", deployContext.CheCluster.Spec.K8s.TlsSecretName)
-			}
-			if _, exists := cheTLSSecret.Data["tls.crt"]; !exists {
-				return nil, fmt.Errorf("%s secret has no 'tls.crt' key in data", deployContext.CheCluster.Spec.K8s.TlsSecretName)
-			}
-			k8sCheEnv["CHE_INFRA_KUBERNETES_TLS__KEY"] = string(cheTLSSecret.Data["tls.key"])
-			k8sCheEnv["CHE_INFRA_KUBERNETES_TLS__CERT"] = string(cheTLSSecret.Data["tls.crt"])
 		}
 
 		addMap(cheEnv, k8sCheEnv)
 	}
 
 	addMap(cheEnv, deployContext.CheCluster.Spec.Server.CustomCheProperties)
+
+	// Update BitBucket endpoints
+	secrets, err := deploy.GetSecrets(deployContext, map[string]string{
+		deploy.KubernetesPartOfLabelKey:    deploy.CheEclipseOrg,
+		deploy.KubernetesComponentLabelKey: deploy.OAuthScmConfiguration,
+	}, map[string]string{
+		deploy.CheEclipseOrgOAuthScmServer: "bitbucket",
+	})
+	if err != nil {
+		return nil, err
+	} else if len(secrets) == 1 {
+		serverEndpoint := secrets[0].Annotations[deploy.CheEclipseOrgScmServerEndpoint]
+		endpoints, exists := cheEnv["CHE_INTEGRATION_BITBUCKET_SERVER__ENDPOINTS"]
+		if exists {
+			cheEnv["CHE_INTEGRATION_BITBUCKET_SERVER__ENDPOINTS"] = endpoints + "," + serverEndpoint
+		} else {
+			cheEnv["CHE_INTEGRATION_BITBUCKET_SERVER__ENDPOINTS"] = serverEndpoint
+		}
+	}
+
 	return cheEnv, nil
 }
 
 func GetCheConfigMapVersion(deployContext *deploy.DeployContext) string {
-	cheConfigMap, _ := deploy.GetClusterConfigMap("che", deployContext.CheCluster.Namespace, deployContext.ClusterAPI.Client)
-	if cheConfigMap != nil {
+	cheConfigMap := &corev1.ConfigMap{}
+	exists, _ := deploy.GetNamespacedObject(deployContext, CheConfigMapName, cheConfigMap)
+	if exists {
 		return cheConfigMap.ResourceVersion
 	}
 	return ""
