@@ -13,6 +13,7 @@ package util
 
 import (
 	"bytes"
+	"context"
 	"crypto/tls"
 	"encoding/json"
 	"errors"
@@ -28,14 +29,18 @@ import (
 	"strings"
 	"time"
 
-	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
+	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
 	"github.com/sirupsen/logrus"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/discovery"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/config"
+	"sigs.k8s.io/yaml"
 )
 
 var (
@@ -139,12 +144,30 @@ func getApiList() ([]v1.APIGroup, error) {
 	return apiList.Groups, nil
 }
 
-func GetServerResources() ([]*v1.APIResourceList, error) {
+func HasAPIResourceName(name string) bool {
 	discoveryClient, err := getDiscoveryClient()
 	if err != nil {
-		return nil, err
+		return false
 	}
-	return discoveryClient.ServerResources()
+
+	_, resourcesList, err := discoveryClient.ServerGroupsAndResources()
+	if err != nil {
+		return false
+	}
+
+	return HasAPIResourceNameInList(name, resourcesList)
+}
+
+func HasAPIResourceNameInList(name string, resources []*metav1.APIResourceList) bool {
+	for _, l := range resources {
+		for _, r := range l.APIResources {
+			if r.Name == name {
+				return true
+			}
+		}
+	}
+
+	return false
 }
 
 func GetValue(key string, defaultValue string) (value string) {
@@ -355,6 +378,14 @@ func IsOAuthEnabled(c *orgv1.CheCluster) bool {
 	return false
 }
 
+// IsInitialOpenShiftOAuthUserEnabled returns true when initial Openshift oAuth user is enabled for CheCluster resource, otherwise false.
+func IsInitialOpenShiftOAuthUserEnabled(c *orgv1.CheCluster) bool {
+	if c.Spec.Auth.InitialOpenShiftOAuthUser != nil && *c.Spec.Auth.InitialOpenShiftOAuthUser {
+		return true
+	}
+	return false
+}
+
 // IsWorkspaceInSameNamespaceWithChe return true when Che workspaces will be executed in the same namespace with Che, otherwise returns false.
 func IsWorkspaceInSameNamespaceWithChe(cr *orgv1.CheCluster) bool {
 	return GetWorkspaceNamespaceDefault(cr) == cr.Namespace
@@ -376,9 +407,46 @@ func GetWorkspaceNamespaceDefault(cr *orgv1.CheCluster) string {
 	return GetValue(cr.Spec.Server.WorkspaceNamespaceDefault, workspaceNamespaceDefault)
 }
 
+// IsDeleteOAuthInitialUser - returns true when initial Openshfit oAuth user must be deleted.
+func IsDeleteOAuthInitialUser(cr *orgv1.CheCluster) bool {
+	return cr.Spec.Auth.InitialOpenShiftOAuthUser != nil && !*cr.Spec.Auth.InitialOpenShiftOAuthUser && cr.Status.OpenShiftOAuthUserCredentialsSecret != ""
+}
+
 func GetResourceQuantity(value string, defaultValue string) resource.Quantity {
 	if value != "" {
 		return resource.MustParse(value)
 	}
 	return resource.MustParse(defaultValue)
 }
+
+// Finds Env by a given name
+func FindEnv(envs []corev1.EnvVar, name string) *corev1.EnvVar {
+	for _, env := range envs {
+		if env.Name == name {
+			return &env
+		}
+	}
+
+	return nil
+}
+
+func ReadObject(yamlFile string, obj interface{}) error {
+	data, err := ioutil.ReadFile(yamlFile)
+	if err != nil {
+		return err
+	}
+
+	err = yaml.Unmarshal(data, obj)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func ReloadCheCluster(client client.Client, cheCluster *orgv1.CheCluster) error {
+	return client.Get(
+		context.TODO(),
+		types.NamespacedName{Name: cheCluster.Name, Namespace: cheCluster.Namespace},
+		cheCluster)
+}
